@@ -16,9 +16,9 @@
 #define blip_add_delta blip_add_delta_fast
 #endif
 
-#define FRAME_CYCLES (DMG_LR35902_FREQUENCY >> 9)
+#define FRAME_CYCLES (DMG_SM83_FREQUENCY >> 9)
 
-const uint32_t DMG_LR35902_FREQUENCY = 0x400000;
+const uint32_t DMG_SM83_FREQUENCY = 0x400000;
 static const int CLOCKS_PER_BLIP_FRAME = 0x1000;
 static const unsigned BLIP_BUFFER_SIZE = 0x4000;
 const int GB_AUDIO_VOLUME_MAX = 0x100;
@@ -37,7 +37,7 @@ static bool _updateSweep(struct GBAudioSquareChannel* sweep, bool initial);
 static void _updateSquareSample(struct GBAudioSquareChannel* ch);
 static int32_t _updateSquareChannel(struct GBAudioSquareChannel* ch);
 
-static int8_t _coalesceNoiseChannel(struct GBAudioNoiseChannel* ch);
+static int16_t _coalesceNoiseChannel(struct GBAudioNoiseChannel* ch);
 
 static void _updateFrame(struct mTiming* timing, void* user, uint32_t cyclesLate);
 static void _updateChannel1(struct mTiming* timing, void* user, uint32_t cyclesLate);
@@ -51,10 +51,10 @@ void GBAudioInit(struct GBAudio* audio, size_t samples, uint8_t* nr52, enum GBAu
 	audio->samples = samples;
 	audio->left = blip_new(BLIP_BUFFER_SIZE);
 	audio->right = blip_new(BLIP_BUFFER_SIZE);
-	audio->clockRate = DMG_LR35902_FREQUENCY;
+	audio->clockRate = DMG_SM83_FREQUENCY;
 	// Guess too large; we hang producing extra samples if we guess too low
-	blip_set_rates(audio->left, DMG_LR35902_FREQUENCY, 96000);
-	blip_set_rates(audio->right, DMG_LR35902_FREQUENCY, 96000);
+	blip_set_rates(audio->left, DMG_SM83_FREQUENCY, 96000);
+	blip_set_rates(audio->right, DMG_SM83_FREQUENCY, 96000);
 	audio->forceDisableCh[0] = false;
 	audio->forceDisableCh[1] = false;
 	audio->forceDisableCh[2] = false;
@@ -279,6 +279,7 @@ void GBAudioWriteNR24(struct GBAudio* audio, uint8_t value) {
 void GBAudioWriteNR30(struct GBAudio* audio, uint8_t value) {
 	audio->ch3.enable = GBAudioRegisterBankGetEnable(value);
 	if (!audio->ch3.enable) {
+		mTimingDeschedule(audio->timing, &audio->ch3Event);
 		audio->playingCh3 = false;
 		*audio->nr52 &= ~0x0004;
 	}
@@ -329,7 +330,9 @@ void GBAudioWriteNR34(struct GBAudio* audio, uint8_t value) {
 			}
 		}
 		audio->ch3.window = 0;
-		audio->ch3.sample = 0;
+		if (audio->style == GB_AUDIO_DMG) {
+			audio->ch3.sample = 0;
+		}
 	}
 	mTimingDeschedule(audio->timing, &audio->ch3Fade);
 	mTimingDeschedule(audio->timing, &audio->ch3Event);
@@ -356,6 +359,7 @@ void GBAudioWriteNR42(struct GBAudio* audio, uint8_t value) {
 }
 
 void GBAudioWriteNR43(struct GBAudio* audio, uint8_t value) {
+	// TODO: Reschedule event
 	audio->ch4.ratio = GBAudioRegisterNoiseFeedbackGetRatio(value);
 	audio->ch4.frequency = GBAudioRegisterNoiseFeedbackGetFrequency(value);
 	audio->ch4.power = GBAudioRegisterNoiseFeedbackGetPower(value);
@@ -510,6 +514,9 @@ void GBAudioUpdateFrame(struct GBAudio* audio, struct mTiming* timing) {
 				audio->playingCh1 = _updateSweep(&audio->ch1, false);
 				*audio->nr52 &= ~0x0001;
 				*audio->nr52 |= audio->playingCh1;
+				if (!audio->playingCh1) {
+					mTimingDeschedule(audio->timing, &audio->ch1Event);
+				}
 			}
 		}
 		// Fall through
@@ -596,7 +603,7 @@ void GBAudioSamplePSG(struct GBAudio* audio, int16_t* left, int16_t* right) {
 	int sampleLeft = dcOffset;
 	int sampleRight = dcOffset;
 
-	if (audio->playingCh1 && !audio->forceDisableCh[0]) {
+	if (!audio->forceDisableCh[0]) {
 		if (audio->ch1Left) {
 			sampleLeft += audio->ch1.sample;
 		}
@@ -606,7 +613,7 @@ void GBAudioSamplePSG(struct GBAudio* audio, int16_t* left, int16_t* right) {
 		}
 	}
 
-	if (audio->playingCh2 && !audio->forceDisableCh[1]) {
+	if (!audio->forceDisableCh[1]) {
 		if (audio->ch2Left) {
 			sampleLeft +=  audio->ch2.sample;
 		}
@@ -616,7 +623,7 @@ void GBAudioSamplePSG(struct GBAudio* audio, int16_t* left, int16_t* right) {
 		}
 	}
 
-	if (audio->playingCh3 && !audio->forceDisableCh[2]) {
+	if (!audio->forceDisableCh[2]) {
 		if (audio->ch3Left) {
 			sampleLeft += audio->ch3.sample;
 		}
@@ -626,8 +633,11 @@ void GBAudioSamplePSG(struct GBAudio* audio, int16_t* left, int16_t* right) {
 		}
 	}
 
-	if (audio->playingCh4 && !audio->forceDisableCh[3]) {
-		int8_t sample = _coalesceNoiseChannel(&audio->ch4);
+	sampleLeft <<= 3;
+	sampleRight <<= 3;
+
+	if (!audio->forceDisableCh[3]) {
+		int16_t sample = audio->style == GB_AUDIO_GBA ? (audio->ch4.sample << 3) : _coalesceNoiseChannel(&audio->ch4);
 		if (audio->ch4Left) {
 			sampleLeft += sample;
 		}
@@ -637,9 +647,6 @@ void GBAudioSamplePSG(struct GBAudio* audio, int16_t* left, int16_t* right) {
 		}
 	}
 
-	sampleLeft <<= 3;
-	sampleRight <<= 3;
-
 	*left = sampleLeft * (1 + audio->volumeLeft);
 	*right = sampleRight * (1 + audio->volumeRight);
 }
@@ -762,12 +769,12 @@ static int32_t _updateSquareChannel(struct GBAudioSquareChannel* ch) {
 	}
 }
 
-static int8_t _coalesceNoiseChannel(struct GBAudioNoiseChannel* ch) {
+static int16_t _coalesceNoiseChannel(struct GBAudioNoiseChannel* ch) {
 	if (!ch->nSamples) {
-		return ch->sample;
+		return ch->sample << 3;
 	}
 	// TODO keep track of timing
-	int8_t sample = ch->samples / ch->nSamples;
+	int16_t sample = (ch->samples << 3) / ch->nSamples;
 	ch->nSamples = 0;
 	ch->samples = 0;
 	return sample;
@@ -927,14 +934,32 @@ static void _updateChannel4(struct mTiming* timing, void* user, uint32_t cyclesL
 	cycles <<= ch->frequency;
 	cycles *= 8 * audio->timingFactor;
 
-	int lsb = ch->lfsr & 1;
-	ch->sample = lsb * ch->envelope.currentVolume;
-	++ch->nSamples;
-	ch->samples += ch->sample;
-	ch->lfsr >>= 1;
-	ch->lfsr ^= (lsb * 0x60) << (ch->power ? 0 : 8);
+	uint32_t last = 0;
+	uint32_t now = cycles;
+	int32_t next = cycles - cyclesLate;
 
-	mTimingSchedule(timing, &audio->ch4Event, cycles - cyclesLate);
+	if (audio->style == GB_AUDIO_GBA) {
+		last = ch->lastEvent;
+		now = mTimingCurrentTime(timing) - cyclesLate;
+		ch->lastEvent = now;
+		now -= last;
+		last = 0;
+		if (audio->sampleInterval > next) {
+			// TODO: Make batching work when descheduled
+			next = audio->sampleInterval;
+		}
+	}
+
+	for (; last < now; last += cycles) {
+		int lsb = ch->lfsr & 1;
+		ch->sample = lsb * ch->envelope.currentVolume;
+		++ch->nSamples;
+		ch->samples += ch->sample;
+		ch->lfsr >>= 1;
+		ch->lfsr ^= (lsb * 0x60) << (ch->power ? 0 : 8);
+	}
+
+	mTimingSchedule(timing, &audio->ch4Event, next);
 }
 
 void GBAudioPSGSerialize(const struct GBAudio* audio, struct GBSerializedPSGState* state, uint32_t* flagsOut) {
@@ -978,6 +1003,7 @@ void GBAudioPSGSerialize(const struct GBAudio* audio, struct GBSerializedPSGStat
 	ch4Flags = GBSerializedAudioEnvelopeSetLength(ch4Flags, audio->ch4.length);
 	ch4Flags = GBSerializedAudioEnvelopeSetNextStep(ch4Flags, audio->ch4.envelope.nextStep);
 	STORE_32LE(ch4Flags, 0, &state->ch4.envelope);
+	STORE_32LE(audio->ch4.lastEvent, 0, &state->ch4.lastEvent);
 	STORE_32LE(audio->ch4Event.when - mTimingCurrentTime(audio->timing), 0, &state->ch4.nextEvent);
 
 	STORE_32LE(flags, 0, flagsOut);
@@ -1049,8 +1075,17 @@ void GBAudioPSGDeserialize(struct GBAudio* audio, const struct GBSerializedPSGSt
 	audio->ch4.length = GBSerializedAudioEnvelopeGetLength(ch4Flags);
 	audio->ch4.envelope.nextStep = GBSerializedAudioEnvelopeGetNextStep(ch4Flags);
 	LOAD_32LE(audio->ch4.lfsr, 0, &state->ch4.lfsr);
+	LOAD_32LE(audio->ch4.lastEvent, 0, &state->ch4.lastEvent);
 	LOAD_32LE(when, 0, &state->ch4.nextEvent);
 	if (audio->ch4.envelope.dead < 2 && audio->playingCh4) {
+		if (!audio->ch4.lastEvent) {
+			// Back-compat: fake this value
+			uint32_t currentTime = mTimingCurrentTime(audio->timing);
+			int32_t cycles = audio->ch4.ratio ? 2 * audio->ch4.ratio : 1;
+			cycles <<= audio->ch4.frequency;
+			cycles *= 8 * audio->timingFactor;
+			audio->ch4.lastEvent = currentTime + (when & (cycles - 1)) - cycles;
+		}
 		mTimingSchedule(audio->timing, &audio->ch4Event, when);
 	}
 }
